#!/bin/sh
# the next line restarts using tclsh \
exec tclsh "$0" "$@"

namespace eval JO3EMC {

# 基本事項初期化
proc setVN {key value} {variable VN; set VN($key) [subst $value]}
proc initializeBasic {} {
  lmap i {VN} {variable $i}
  array unset VN
  
  # 設定値初期化
  setVN  information.title                {EMC Clock}
  setVN  information.version              {20200524}
  setVN  information.copyright            {(c) 2012 JO3EMC}
  setVN  information.developer            {岩橋 伴典 (JO3EMC)}
  setVN  information.template             {JO3EMC 20200520.tcl}
  setVN  encoding.script                  {utf-8}
  setVN  filepath.script.main             {[file join $::argv0]}
  setVN  filepath:folder.script.main      {[file dirname $VN(filepath.script.main)]}
  setVN  filepath:folder.root             {$VN(filepath:folder.script.main)}
  setVN  filepath.source.setting          {./setting.txt}
  setVN  console.show                     {0}
  setVN  constant.pi                      {[expr 4*atan(1)]}
  setVN  delay.loop                       {50}
  setVN  fontname                         {Helvetica}
  setVN  scale.font.analog                {0.15}
  setVN  scale.font.digital.date          {0.15}
  setVN  scale.font.digital.time          {0.25}
  setVN  scale.font.digital.DotW          {0.10}
  setVN  color.digital.date               {gray}
  setVN  color.digital.time               {white}
  setVN  color.digital.DotW               {gray}
  setVN  scale.font.platform              {1.0} ;# 文字倍率 環境別調整用 標準
  setVN  scale.font.platform.AndroWish    {0.2} ;# 文字倍率 環境別調整用 AndroWish
  
  # 外部基本項目設定読込
  set VN(filepath.source.setting) [file normalize [file join $VN(filepath:folder.root) $VN(filepath.source.setting)]]
  if {[file isfile $VN(filepath.source.setting)]} then {source -encoding $VN(encoding.script) $VN(filepath.source.setting)}
  
  # 環境別調整用フォント倍率修正
  if {![catch {package require borg}]} then {set VN(scale.font.platform) $VN(scale.font.platform.AndroWish)}
  
}

# GUI初期化
proc ToggleFullScreen {window} {wm attributes $window -fullscreen [expr ![wm attributes $window -fullscreen]]}
proc initializeGUI {{mode {A}}} {
  lmap i {VN W} {variable $i}
  array unset W
  
  # トップレベルウインドウ設定
  set W(toplevel.main) .
  set VT(window) $W(toplevel.main)
  wm title $VT(window) "$VN(information.title) Ver.$VN(information.version)"
#  wm resizable $VT(window) 0 0
##  wm state $VT(window) zoomed
#  wm geometry $VT(window) "[join [wm maxsize .] x]+0+0"
#  wm attributes $VT(window) -fullscreen 1
  
  # ウィジェット構築
  lmap i {display} {
    set VT(window) [string trimright $W(toplevel.main) {.}].$i
    catch {destroy $VT(window)}
    set W(frame.$i) [frame $VT(window)]
    pack $W(frame.$i) -expand yes -fill both
  }
  
  # 主表示画面用 キャンバスウィジェット
  set W(canvas) $W(frame.display).canvas
  pack [canvas $W(canvas)] -fill both -expand yes
  $W(canvas) configure -bg black -highlightthickness 0
  
  # タッチパネル操作用 モード切替ボタンアイテム
  set W(button.setting) $W(canvas).bSetting
  set VT(list) {A D}
  set i [expr ([lsearch $VT(list) $mode]+1)%[llength $VT(list)]]
  set i [lindex $VT(list) $i]
  button $W(button.setting) -text "$i" -command "[namespace current]::initializeGUI $i"
  $W(canvas) create window {0 0} -window $W(button.setting) -anchor nw
  # 操作ヒントテキストアイテム構築
  $W(canvas) create text {0 0} -tags "note.0" -fill gray -anchor se -justify right -text "Press <F11> or <Click>\n to toggle FullScreen."
  
  # 時計表示アイテム（アナログ表示）
  if {$mode eq {A}} then {
    foreach i {0 1} {$W(canvas) create oval {0 0 0 0} -tags "dial.$i" -fill [lindex {gray white} $i] -width 0}
    for {set i 1} {$i <= 60} {incr i} {$W(canvas) create line {0 0 0 0} -tags "tic.$i"}
    for {set i 1} {$i <= 12} {incr i} {$W(canvas) create text {0 0}     -tags "numeral.$i" -text $i}
    foreach i {0 1 2} {$W(canvas) create line {0 0 0 0} -tags "hand.$i" -capstyle round -fill [lindex {black black red} $i]}
  }
  # 時計表示アイテム（デジタル表示）
  if {$mode eq {D}} then {
    foreach i {0 1 2} {$W(canvas) create text {0 0} -tags "digital.$i"}
  }
  
  # バインディング構築
  lmap VT(key) {toplevel.main} {
    set VT(window) $W($VT(key))
    bind $VT(window) <Alt-Key-F4>    "exit"
    bind $VT(window) <Key-Escape>    "exit"
    bind $VT(window) <Key-F1>        "console hide; console show"
    bind $VT(window) <Key-F2>        "console hide"
    bind $VT(window) <Key-F11>       "[namespace current]::ToggleFullScreen $W(toplevel.main)"
    lmap i {a A d D} {
      bind $VT(window) <Key-$i> "[namespace current]::initializeGUI [string toupper $i]"
    }
  }
  set VT(window) $W(canvas)
  bind $VT(window) <ButtonPress-1> "[namespace current]::ToggleFullScreen $W(toplevel.main)"
  bind $VT(window) <Configure>     "unset -nocomplain [namespace current]::VN(flag.reformed)"
  
  # 補足処理
  if {[info exists VN(console.show)] && ($VN(console.show) != 0)} then {console show}
  focus -force $W(toplevel.main)
  update
}


# ループ処理本体
proc body {} {
  foreach i {VN W} {variable $i}
  
  # キャンバスサイズによるパラメーター設定
  set VL(wh.canvas) [lmap i {width height} {winfo $i $W(canvas)}]
  set VL(xy.center) [lmap i $VL(wh.canvas) {expr $i/2}]
  lassign $VL(xy.center) VL(x.center) VL(y.center)
  set VL(radius) [expr min([join $VL(xy.center) {,}])]
  
  # キャンバス状態に変化があった場合の処理
  if {![info exists VN(flag.reformed)]} {
    # アナログ表示 文字盤 枠
    set VL(list.scale.dial) {1 0.95}
    foreach i {0 1} {
      set VT(radius) [expr $VL(radius)*[lindex $VL(list.scale.dial) $i]]
      $W(canvas) coords "dial.$i" [concat [lmap j $VL(xy.center) {expr $j-$VT(radius)}] [lmap j $VL(xy.center) {expr $j+$VT(radius)}]]
    }
    $W(canvas) itemconfigure "dial.1" -width [expr $VL(radius)*0.01]
    
    # アナログ表示 文字盤 目盛り
    for {set i 1} {$i <= 60} {incr i} {
      set VT(coords) {}
      set VT(rad) [expr 2*$VN(constant.pi)*$i/60]
      set VT(list.scale.distance) [list [lindex $VL(list.scale.dial) 1] {0.9}]
      if {![expr $i%5]} then {lset VT(list.scale.distance) 1 0.85}
      foreach j {0 1} {
        set VT(distance) [expr $VL(radius)*[lindex $VT(list.scale.distance) $j]]
        lappend VT(coords) [expr $VL(x.center)+$VT(distance)*sin($VT(rad))]
        lappend VT(coords) [expr $VL(y.center)-$VT(distance)*cos($VT(rad))]
      }
      $W(canvas) coords "tic.$i" $VT(coords)
      set VT(width) {0.015}
      if {![expr $i%5]} then {set VT(width) {0.03}}
      $W(canvas) itemconfigure "tic.$i" -width [expr $VL(radius)*$VT(width)]
    }
    
    # アナログ表示 文字盤 数字
    set VT(distance) [expr $VL(radius)*0.7]
    for {set i 1} {$i <= 12} {incr i} {
      set VT(coords) {}
      set VT(rad) [expr 2*$VN(constant.pi)*$i/12]
      lappend VT(coords) [expr $VL(x.center)+$VT(distance)*sin($VT(rad))]
      lappend VT(coords) [expr $VL(y.center)-$VT(distance)*cos($VT(rad))]
      $W(canvas) coords "numeral.$i" $VT(coords)
      $W(canvas) itemconfigure "numeral.$i" -font [list $VN(fontname) [expr round($VL(radius)*$VN(scale.font.platform)*$VN(scale.font.analog))] {bold}]
    }
    
    # デジタル表示
    foreach i {0 1 2} {
      set j [lindex {date time DotW} $i]
      $W(canvas) coords "digital.$i" [list $VL(x.center) [expr $VL(y.center)-$VL(radius)*[lindex {+0.45 -0.45 +0.1} $i]]]
      $W(canvas) itemconfigure "digital.$i" -fill $VN(color.digital.$j) -font [list $VN(fontname) [expr round([lindex $VL(wh.canvas) 0]*0.5*$VN(scale.font.platform)*$VN(scale.font.digital.$j))] {bold}]
    }
    
    # アナログ表示 針
    foreach i {0 1 2} {
      $W(canvas) itemconfigure "hand.$i" -width [expr $VL(radius)*[lindex {0.1 0.05 0.02} $i]]
    }
    
    # 操作ヒントテキスト
    $W(canvas) coords "note.0" $VL(wh.canvas)
    $W(canvas) itemconfigure "note.0" -font [list $VN(fontname) [expr round($VL(radius)*$VN(scale.font.platform)*0.03)]]
    
    # キャンバス状態変化フラグ設定
    set VN(flag.reformed) 1
  }
  
  # 現在時刻取得
  set VL(clock) [clock seconds]
  
  # 表示時刻更新（アナログ表示）
  lmap i [clock format $VL(clock) -format "%H %M %S"] j {hour minute second} {set VT($j) [scan $i {%d}]}
  set VT(ratio.hour) [expr (($VT(hour)%12)+$VT(minute)/60.0+$VT(second)/3600.0)/12.0]
  foreach i {minute second} {set VT(ratio.$i) [expr $VT($i)/60.0]}
  foreach i {0 1 2} {
    set VT(key) [lindex {hour minute second} $i]
    set VT(coords) $VL(xy.center)
    set VT(rad) [expr 2*$VN(constant.pi)*$VT(ratio.$VT(key))]
    set VT(distance) [expr $VL(radius)*[lindex {0.55 0.75 0.85} $i]]
    lappend VT(coords) [expr $VL(x.center)+$VT(distance)*sin($VT(rad))]
    lappend VT(coords) [expr $VL(y.center)-$VT(distance)*cos($VT(rad))]
    $W(canvas) coords "hand.$i" $VT(coords)
  }
  
  # 表示時刻更新（デジタル表示）
  $W(canvas) itemconfigure "digital.0" -text "[clock format $VL(clock) -format {%Y-%m-%d}]"
  $W(canvas) itemconfigure "digital.1" -text "[clock format $VL(clock) -format {%H:%M:%S}]"
  $W(canvas) itemconfigure "digital.2" -text "[clock format $VL(clock) -format {(%a)}]"
  
  # 表示状態更新&ループ処理
  update
  set VN(after) [after $VN(delay.loop) "namespace eval [namespace current] body"]
}


# メインプロセス
proc main {} {
  lmap i {VN} {variable $i}
  
  # 基本事項初期化
  initializeBasic
  
  # GUI初期化
  if {![catch {package present Tk}]} then {initializeGUI}
  
  # ループ処理
  body
  
}
main

}
